kedro.extras.datasets.yaml.yaml_dataset — Kedro 0.17.6 documentation

# Source code for kedro.extras.datasets.yaml.yaml_dataset

    """``YAMLDataSet`` loads/saves data from/to a YAML file using an underlying
    filesystem (e.g.: local, S3, GCS). It uses PyYAML to handle the YAML file.
    """
    from copy import deepcopy
    from pathlib import PurePosixPath
    from typing import Any, Dict, Union

    import fsspec
    import pandas as pd
    import yaml

    from kedro.io.core import (
        AbstractVersionedDataSet,
        DataSetError,
        Version,
        get_filepath_str,
        get_protocol_and_path,
    )

    class YAMLDataSet(AbstractVersionedDataSet):
        """``YAMLDataSet`` loads/saves data from/to a YAML file using an underlying
        filesystem (e.g.: local, S3, GCS). It uses PyYAML to handle the YAML file.

        Example:
        ::

            >>> from kedro.extras.datasets.yaml import YAMLDataSet
            >>>
            >>> data = {'col1': [1, 2], 'col2': [4, 5], 'col3': [5, 6]}
            >>>
            >>> # data_set = YAMLDataSet(filepath="gcs://bucket/test.yaml")
            >>> data_set = YAMLDataSet(filepath="test.yaml")
            >>> data_set.save(data)
            >>> reloaded = data_set.load()
            >>> assert data == reloaded

        """

        DEFAULT_SAVE_ARGS = {"default_flow_style": False}  # type: Dict[str, Any]

        # pylint: disable=too-many-arguments
        def __init__(
            self,
            filepath: str,
            save_args: Dict[str, Any] = None,
            version: Version = None,
            credentials: Dict[str, Any] = None,
            fs_args: Dict[str, Any] = None,
        ) -> None:
            """Creates a new instance of ``YAMLDataSet`` pointing to a concrete YAML file
            on a specific filesystem.

            Args:
                filepath: Filepath in POSIX format to a YAML file prefixed with a protocol like `s3://`.
                    If prefix is not provided, `file` protocol (local filesystem) will be used.
                    The prefix should be any protocol supported by ``fsspec``.
                    Note: `http(s)` doesn't support versioning.
                save_args: PyYAML options for saving YAML files (arguments passed
                    into ```yaml.dump``). Here you can find all available arguments:
                    https://pyyaml.org/wiki/PyYAMLDocumentation
                    All defaults are preserved, but "default_flow_style", which is set to False.
                version: If specified, should be an instance of
                    ``kedro.io.core.Version``. If its ``load`` attribute is
                    None, the latest version will be loaded. If its ``save``
                    attribute is None, save version will be autogenerated.
                credentials: Credentials required to get access to the underlying filesystem.
                    E.g. for ``GCSFileSystem`` it should look like `{"token": None}`.
                fs_args: Extra arguments to pass into underlying filesystem class constructor
                    (e.g. `{"project": "my-project"}` for ``GCSFileSystem``), as well as
                    to pass to the filesystem's `open` method through nested keys
                    `open_args_load` and `open_args_save`.
                    Here you can find all available arguments for `open`:
                    https://filesystem-spec.readthedocs.io/en/latest/api.html#fsspec.spec.AbstractFileSystem.open
                    All defaults are preserved, except `mode`, which is set to `r` when loading
                    and to `w` when saving.
            """
            _fs_args = deepcopy(fs_args) or {}
            _fs_open_args_load = _fs_args.pop("open_args_load", {})
            _fs_open_args_save = _fs_args.pop("open_args_save", {})
            _credentials = deepcopy(credentials) or {}

            protocol, path = get_protocol_and_path(filepath, version)
            if protocol == "file":
                _fs_args.setdefault("auto_mkdir", True)

            self._protocol = protocol
            self._fs = fsspec.filesystem(self._protocol, **_credentials, **_fs_args)

            super().__init__(
                filepath=PurePosixPath(path),
                version=version,
                exists_function=self._fs.exists,
                glob_function=self._fs.glob,
            )

            # Handle default save arguments
            self._save_args = deepcopy(self.DEFAULT_SAVE_ARGS)
            if save_args is not None:
                self._save_args.update(save_args)

            _fs_open_args_save.setdefault("mode", "w")
            self._fs_open_args_load = _fs_open_args_load
            self._fs_open_args_save = _fs_open_args_save

        def _describe(self) -> Dict[str, Any]:
            return dict(
                filepath=self._filepath,
                protocol=self._protocol,
                save_args=self._save_args,
                version=self._version,
            )

        def _load(self) -> Dict:
            load_path = get_filepath_str(self._get_load_path(), self._protocol)

            with self._fs.open(load_path, **self._fs_open_args_load) as fs_file:
                return yaml.safe_load(fs_file)

        def _save(self, data: Union[Dict, pd.DataFrame]) -> None:
            save_path = get_filepath_str(self._get_save_path(), self._protocol)

            if isinstance(data, pd.DataFrame):
                data = data.to_dict()

            with self._fs.open(save_path, **self._fs_open_args_save) as fs_file:
                yaml.dump(data, fs_file, **self._save_args)

            self._invalidate_cache()

        def _exists(self) -> bool:
            try:
                load_path = get_filepath_str(self._get_load_path(), self._protocol)
            except DataSetError:
                return False

            return self._fs.exists(load_path)

        def _release(self) -> None:
            super()._release()
            self._invalidate_cache()

        def _invalidate_cache(self) -> None:
            """Invalidate underlying filesystem caches."""
            filepath = get_filepath_str(self._filepath, self._protocol)
            self._fs.invalidate_cache(filepath)